94057c3d3e
PR Test (NPU) / check-changes (push) Has been cancelled
PR Test (NPU) / pr-gate (push) Has been cancelled
PR Test (NPU) / set-image-config (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-1-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (0) (push) Has been cancelled
PR Test (NPU) / stage-b-test-2-npu-a2 (1) (push) Has been cancelled
PR Test (NPU) / stage-b-test-4-npu-a3 (push) Has been cancelled
PR Test (NPU) / stage-b-test-16-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-1-npu-a3 (push) Has been cancelled
PR Test (NPU) / multimodal-gen-test-2-npu-a3 (push) Has been cancelled
PR Test (Arm64) / pr-gate (push) Has been cancelled
PR Test (Arm64) / check-changes (push) Has been cancelled
PR Test (Arm64) / build-test (push) Has been cancelled
PR Test (sgl-router) / gate (push) Has been cancelled
PR Test (sgl-router) / tier-1 — lint (push) Has been cancelled
PR Test (sgl-router) / tier-2 — build + test (push) Has been cancelled
PR Test (sgl-router) / tier-3 — docker (placeholder) (push) Has been cancelled
PR Test (sgl-router) / tier-3 — k8s integration (push) Has been cancelled
PR Test (sgl-router) / tier-3 — e2e (push) Has been cancelled
PR Test (sgl-router) / finish (push) Has been cancelled
PR Test (NPU) / single-node-poc (map[name:qwen3_6_27b_w8a8_1p_in64k_out1k_50ms runner:linux-aarch64-a3-2 test_case:test/registered/ascend/performance/qwen3_6_27b/test_npu_qwen3_6_27b_w8a8_1p_in64k_out1k_50ms.py test_type:perf]) (push) Has been cancelled
PR Test (NPU) / pr-test-npu-finish (push) Has been cancelled
PR Test (Xeon) / pr-gate (push) Has been cancelled
PR Test (Xeon) / check-changes (push) Has been cancelled
PR Test (Xeon) / build-test (, xeon-gnr, base-b-test-cpu) (push) Has been cancelled
PR Test (XPU) / check-changes (push) Has been cancelled
PR Test (XPU) / pr-gate (push) Has been cancelled
PR Test (XPU) / stage-a-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / wait-for-stage-a (push) Has been cancelled
PR Test (XPU) / stage-b-test-1-gpu-xpu (push) Has been cancelled
PR Test (XPU) / finish (push) Has been cancelled
CI Model Inventory / build-inventory (push) Has been cancelled
Lint / lint (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Compilation Check (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Manual Policy (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark - Request Processing (push) Has been cancelled
PR Benchmark (SMG Components) / Benchmark Summary (push) Has been cancelled
PR Test (SMG) / build-wheel (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on windows (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (x86_64 - auto) (push) Has been cancelled
PR Test (SMG) / python-unit-tests (push) Has been cancelled
PR Test (SMG) / unit-tests (push) Has been cancelled
PR Test (SMG) / benchmarks (push) Has been cancelled
PR Test (SMG) / chat-completions (push) Has been cancelled
PR Test (SMG) / chat-completions-4gpu (push) Has been cancelled
PR Test (SMG) / e2e (push) Has been cancelled
PR Test (SMG) / docker-build-test (push) Has been cancelled
PR Test (SMG) / k8s-integration (push) Has been cancelled
PR Test (SMG) / finish (push) Has been cancelled
PR Test (SMG) / summarize-benchmarks (push) Has been cancelled
Release SGLang Model Gateway Docker Image / publish (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on macos (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - auto) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (aarch64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / build on linux (x86_64 - musllinux_1_1) (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Build SDist (push) Has been cancelled
Release SGLang Model Gateway to PyPI / Upload to PyPI (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (aarch64, 12.9, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu129-matrix (x86_64, 12.9, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu129 (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (aarch64, 13.0, 3.10, arm-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / build-cu130-matrix (x86_64, 13.0, 3.10, x64-kernel-build-node) (push) Has been cancelled
Release SGLang Kernels / release-cu130 (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 700) (push) Has been cancelled
Release SGLang Kernels / build-rocm-matrix (3.10, 720) (push) Has been cancelled
Release SGLang Kernels / release-rocm700 (push) Has been cancelled
Release SGLang Kernels / release-rocm720 (push) Has been cancelled
Release SGLang Kernels / build-musa43 (43, 3.10) (push) Has been cancelled
Release SGLang Kernels / release-musa43 (push) Has been cancelled
97 lines
2.8 KiB
Python
97 lines
2.8 KiB
Python
from typing import Optional, Tuple
|
|
|
|
import torch
|
|
import triton
|
|
import triton.language as tl
|
|
|
|
|
|
@triton.jit
|
|
def merge_state_kernel(
|
|
output, # [NUM_TOKENS, NUM_HEADS, HEAD_SIZE] v_merged
|
|
output_lse, # [NUM_TOKENS, NUM_HEADS] s_merged
|
|
prefix_output, # [NUM_TOKENS, NUM_HEADS, HEAD_SIZE] v_a
|
|
prefix_lse, # [NUM_TOKENS, NUM_HEADS] s_a
|
|
suffix_output, # [NUM_TOKENS, NUM_HEADS, HEAD_SIZE] v_b
|
|
suffix_lse, # [NUM_TOKENS, NUM_HEADS] s_b
|
|
HEAD_SIZE: tl.constexpr,
|
|
PADDED_HEAD_SIZE: tl.constexpr,
|
|
OUTPUT_LSE: tl.constexpr,
|
|
):
|
|
token_idx = tl.program_id(0)
|
|
num_tokens = tl.num_programs(0)
|
|
head_idx = tl.program_id(1)
|
|
num_heads = tl.num_programs(1)
|
|
|
|
p_lse = tl.load(prefix_lse + token_idx * num_heads + head_idx)
|
|
s_lse = tl.load(suffix_lse + token_idx * num_heads + head_idx)
|
|
p_lse = float("-inf") if p_lse == float("inf") else p_lse
|
|
s_lse = float("-inf") if s_lse == float("inf") else s_lse
|
|
|
|
max_lse = tl.maximum(p_lse, s_lse)
|
|
p_lse = p_lse - max_lse
|
|
s_lse = s_lse - max_lse
|
|
out_se = tl.exp(p_lse) + tl.exp(s_lse)
|
|
|
|
if OUTPUT_LSE:
|
|
out_lse = tl.log(out_se) + max_lse
|
|
tl.store(output_lse + token_idx * num_heads + head_idx, out_lse)
|
|
|
|
head_arange = tl.arange(0, PADDED_HEAD_SIZE)
|
|
head_mask = head_arange < HEAD_SIZE
|
|
p_out = tl.load(
|
|
prefix_output
|
|
+ token_idx * num_heads * HEAD_SIZE
|
|
+ head_idx * HEAD_SIZE
|
|
+ head_arange,
|
|
mask=head_mask,
|
|
)
|
|
s_out = tl.load(
|
|
suffix_output
|
|
+ token_idx * num_heads * HEAD_SIZE
|
|
+ head_idx * HEAD_SIZE
|
|
+ head_arange,
|
|
mask=head_mask,
|
|
)
|
|
|
|
p_scale = tl.exp(p_lse) / out_se
|
|
s_scale = tl.exp(s_lse) / out_se
|
|
out = p_out * p_scale + s_out * s_scale
|
|
tl.store(
|
|
output + token_idx * num_heads * HEAD_SIZE + head_idx * HEAD_SIZE + head_arange,
|
|
out,
|
|
mask=head_mask,
|
|
)
|
|
|
|
|
|
def merge_state_triton(
|
|
prefix_output: torch.Tensor,
|
|
prefix_lse: torch.Tensor,
|
|
suffix_output: torch.Tensor,
|
|
suffix_lse: torch.Tensor,
|
|
output: Optional[torch.Tensor] = None,
|
|
output_lse: Optional[torch.Tensor] = None,
|
|
) -> Tuple[torch.Tensor, Optional[torch.Tensor]]:
|
|
# Avoid creating new tensors if they are already provided
|
|
if output is None:
|
|
output = torch.empty_like(prefix_output)
|
|
if output_lse is None:
|
|
output_lse = torch.empty_like(prefix_lse)
|
|
|
|
num_tokens = output.shape[0]
|
|
num_query_heads = output.shape[1]
|
|
head_size = output.shape[2]
|
|
padded_head_size = triton.next_power_of_2(head_size)
|
|
|
|
merge_state_kernel[(num_tokens, num_query_heads)](
|
|
output,
|
|
output_lse,
|
|
prefix_output,
|
|
prefix_lse,
|
|
suffix_output,
|
|
suffix_lse,
|
|
head_size,
|
|
padded_head_size,
|
|
output_lse is not None,
|
|
)
|
|
return output, output_lse
|